#ifndef CTPLOG_H
#define CTPLOG_H

#if defined(_WINDOWS) || defined(_NTWIN)
#  include <strstrea.h>
#else
#  include <strstream.h>
#endif
#include "ctpdefs.h"
#include "msgtype.h"
#include "ctpfuncs.h"

//
// The purpose of the Log class is to provide a consistant logging
//   interface.  The class will control data formatting, buffering
//   as well as exporting.
//
// Set common log severity

enum Severity {  CTP_INFORMATIONAL=0x10,   CTP_WARNING=0x20,      CTP_USERERROR=0x30,
                 CTP_SEVERE=0x40,          CTP_FATAL=0x50,        CTP_READONLY=0x60,
                 CTPINTERNAL=0x100 };

class EXPORT CTPLog : public ostrstream
{
  public:
    // Enumerated Types
	// Determine which information to put in the header of each message
	enum MsgHeaderInfo { NONE=0x0000,            MESGPREFIX=0x0001,       FILENAME=0x0002,
                             LINENUMBER=0x0004,      TIMESTAMP=0x0008,        LOGGINGLEVEL=0x0010,
                             HOSTMACHINENAME=0x0020, USERID=0x0040,           PROCESSID=0x0080,
		             MESGTYPE=0x0100,        MESGSEVERITY=0x0200,     ALL = 0xFFFF };

        // Determine what severity to log messages above
	enum LogSeverity { LOG_NONE=0x0000, LOG_INFORMATIONAL=CTP_INFORMATIONAL, LOG_WARNING=CTP_WARNING,
			   LOG_USERERROR=CTP_USERERROR, LOG_SEVERE=CTP_SEVERE, LOG_FATAL=CTP_FATAL,
			   LOG_READONLY=CTP_READONLY, LOG_CTPINTERNAL=CTPINTERNAL };

        // Determine what is the severity of the message
	enum MsgSeverity { MSG_NONE=0x0000, MSG_INFORMATIONAL=CTP_INFORMATIONAL, MSG_WARNING=CTP_WARNING,
			   MSG_USERERROR=CTP_USERERROR, MSG_SEVERE=CTP_SEVERE, MSG_FATAL=CTP_FATAL,
			   MSG_READONLY=CTP_READONLY, MSG_CTPINTERNAL=CTPINTERNAL };

    // Construnctors
    CTPLog(ostream *              = NULL,
	   const LogSeverity      = LOG_USERERROR,
    	   const MsgHeaderInfo    = (MsgHeaderInfo)(NONE));

    // Destructors
    virtual ~CTPLog();

    // Public Member functions

    // Get functions

    const CTPString&     getHeaderPrefix()      { return(headerPrefix);  }
    const CTPString&     getMsgFileName()       { return(msgFileName);   }
    const CTPString&     getMsgLineNumber()     { return(msgLineNumber); }
    const MsgSeverity&   getMsgSeverity()       { return(msgSeverity);   }
    const MsgToken&      getMsgToken()          { return(msgToken);      }

    // Set functions
    const MsgHeaderInfo& setHeader(const MsgHeaderInfo);
    const MsgHeaderInfo& setHeader(const int);
    const CTPString&     setHeaderPrefix(const CTPString headerPrefix);

    ostream*             setOutputStream(ostream *ctpOutputStream);

    const LogSeverity&   setLogSeverity(const LogSeverity);
    const LogSeverity&   setLogSeverity(const int);
    const TokenMask&     setLogToken(const MsgToken msgToken);
    const TokenMask&     setLogToken(const TokenMask tokenMask);
    const TokenMask&     addLogToken(const MsgToken msgToken);
    const TokenMask&     rmvLogToken(const MsgToken msgToken);

    const CTPString&     setMsgFileName(const CTPString msgFileName);
    const CTPString&     setMsgLineNumber(const int msgLineNumber);

    const MsgSeverity&   setMsgSeverity(const MsgSeverity);
    const MsgSeverity&   setMsgSeverity(const int);
    const MsgToken&      setMsgToken(const MsgToken msgtype);

    // Overloaded  Operators

 #if !defined(C700)
    CTPLog& operator << (const LogSeverity &);
    CTPLog& operator << (const MsgSeverity &);
    CTPLog& operator << (const MsgToken &);
#endif

    // logError.setMsgFileName(__FILE__), logError.setMsgLineNumber(__LINE__), logError
    CTPLog& operator()(const char *msgFileName="(No Information)", const int msgLineNumber=0);

  private:
    // Member Functions
    streampos           flushOutput();                          // Flush the buffer
    const CTPBoolean&   setActive();                            // Set/unset the active flag

    // Member Data
    CTPBoolean          activeFlag;                             // Is the logging active

    ostream            *ctpOutputStream;                        // Pointer to the output class

    CTPString           headerPrefix;                           // Header prefix CTPString

    MsgHeaderInfo       headerFlags;				// Flags to use for the header developement
    LogSeverity         logSeverity;				// Level at which to log
    MsgSeverity         msgSeverity;				// Severity Level of the last message
    MsgToken            msgToken;		                // Message type of the last message
    TokenMask           logTokenMask;				// Mask what should be printed to log

    CTPString           msgFileName;				// msgFileName & LineNumber must be set
    CTPString           msgLineNumber;				// before each message - else they are null

};

// ----------------------------------------------------------------------
//  provide access for functions outside of support.dll
// ----------------------------------------------------------------------

#if defined(MAC)
CTPLog& getLogError(void);
CTPLog& getLogDebug(void);
#define logDebug   getLogDebug()
#define logError   getLogError()
#else
extern CTPLog& logError;
extern CTPLog& logDebug;
#endif

#define LOGERR			logError(__FILE__,__LINE__)
#define LOGDBG			logDebug(__FILE__,__LINE__)

#define LOGERRORLEVEL(mlevel)	logError.setMsgSeverity(mlevel), LOGERR
#define LOGDEBUGLEVEL(mlevel)	logDebug.setMsgSeverity(mlevel), LOGDBG
#define LOGERRORTYPE(mtype)	logError.setMsgToken(mtype),     LOGERR
#define LOGDEBUGTYPE(mtype)	logDebug.setMsgToken(mtype),     LOGDBG
#define LOGERROR(mtype, mlevel)	logError.setMsgToken(mtype),     LOGERRORLEVEL(mlevel)
#define LOGDEBUG(mtype, mlevel)	logDebug.setMsgToken(mtype),     LOGDEBUGLEVEL(mlevel)

// #define endl "\n"               // Hack to allow Log class to use iostream style output


#endif
