static const char sccsid[]="@(#)rpcmap.cc 1.0.0 2/20/94 (c) Cambridge Technology Partners @(#)";

/***********************************************************************
 * Copyright (C) 1994 Cambridge Technology Partners, all rights reserverd 
 *
 * Module: rpcmap.cc
 * 
 * Description: This module serves the purpose of providing extensions to
 *              the MicroSoft (MS) RPC toolkit. The current release of MS
 *					 RPC toolkit does not provide the functionality to access the
 *					 DCE namespace. This module provides proxy calls to access 
 *					 the DCE name space. The module also provides a Posix thread 
 * 				 interface built on top of Win32 API's thread interface and 
 * 				 synchronization	mechanism.  The module also contains some
 *					 DCE utility functions that are not supported by MS RPC toolkit
 *					 but are essential to the execution of Citibank's C++ Class 
 *					 Library.
 *
 * Created:     Rizwan Mallal, Jan 10th 1994
 * 
 * Modified:    Rizwan Mallal, Jan 18th 1994
 *					 Added a function to provide the mechanism for a client to 
 *					 ping its server.
 * 
 * Modified:    Rizwan Mallal, Jan 25th 1994
 *				   	Added more error checking in the name space calls.
 *
 * Modified:    Rizwan Mallal, Feb 2nd 1994
 *					 Added LOGDBG and LOGERR to provide a better logging
 *					 facility.
 *
 * Modified:    Rizwan Mallal, Feb 10th 1994
 *					 Added the Posix thread interface API calls.
 *
 * Modified:    Rizwan Mallal, Feb 17th 1994
 *					 Added the Posix locking facility for thread synchronization.
 *
 *************************************************************************/

#include <windows.h>
#include <objdce/defs.H>
#include <objdce/bindingDefs.H>
#include <objdce/rpcmap.H>
#include <afx.h>
#include <objdce/binding.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/uuid.H>
#include <objdce/login.H>
#include <proxy.h>


// A binding handle object to the Proxy Server that is 
// running on the Unix host.

Binding * nameBinding = (Binding *)NULL;        

// A synchronization object to protect the critical sections
// of the proxy name space functions.

CriticalSection	      BindingLock;

// A counter to keep track of the threads that are currently
// accessing the nameBinding object.  
int threadCount  = 0;


//
// Function:    FakeRpcNsBindingImportBegin
// Return:      0 on success and an RPC error status on failure
// Description: This function is the first routine called by a DCE 
//					client application to start the process of accessing the 
//					DCE name space. This function will provide the client
//					application with an import context which is actually created 
//					on the Unix side. The import context is the handle
//					used by a client to pass to the CDS daemon.
//
RPC_STATUS RPC_ENTRY FakeRpcNsBindingImportBegin(
			unsigned long    EntryNameSyntax,
			unsigned char *  EntryName,
			RPC_IF_HANDLE    IfSpec,
			UUID *           ObjUuid,
   		RPC_NS_HANDLE *  ImportContext)
{
	error_status_t  status;             // contains the error code
	unsigned long   local_ns_handle;    // access to import context
	unsigned char *  StringUuid = NULL; // string rep. of object uuid

	// Since multiple threads have access to the name binding handle 
	// object we want to make sure the current thread is the sole
	// user of this binding object. 
 
	++BindingLock;    // Block if can't gain access

	// Another thread starting the process of accessing the name
	// space.

	threadCount++;

	// First connect to our proxyServer. Another thread might have
	// already created a name binding handle object. We don't need
	// to duplicate the effort. 

	if (!nameBinding)  
	    nameBinding = new Binding(ProxyServerName, ProxyServerPort, 
										  (LoginContext*)0, RPCClient(Proxy,0,0));
      
	if ((nameBinding->IsValid) == False) {
		status = RPC_S_NAME_SERVICE_UNAVAILABLE;
		CheckStatus(status, true, ErrorLevels::FatalMsg,
			   "Failed Binding to ProxyServer",
			   __LINE__, __FILE__);
		--BindingLock;
		return status;
	}

	//
	// It is quite possible that the application uses an object uuid
	// as an additional parameter to locate its server.
	// Since an object uuid is of an opaque type, we want to 
	// convert it into its string representation before we send it out
	// to our proxy server.
	//

	if (ObjUuid) {	
		status = UuidToString(ObjUuid, &StringUuid); 
		CheckStatus(status, true, ErrorLevels::FatalMsg,
				"UuidToString",
				__LINE__, __FILE__); 
	} else 
		// Since the object uuid is NULL. We don't want to
		// send a null string. This will cause problems in
		// the RPC run-time library. Just pass an empty 
		// string indicating a NULL object uuid.

		StringUuid = (unsigned char *)"";

	// RPC call to the proxy server

	status = RpcNsBindingImportBegin_(
					  *nameBinding,
					  EntryNameSyntax,
					  EntryName,
					  StringUuid,
					  &local_ns_handle);

	if (!CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcNsBindingImportBegin_",
		  __LINE__, __FILE__)) {
		--BindingLock;
		return status;
	}

	
	// Free the string representation of the object uuid since
	// it is the application's responsibility to free all
	// the memory that was allocated by the RPC run-time
	// for string uuid.
	
	if (StringUuid) 
		status = RpcStringFree(&StringUuid);
	
	// 
	// Now we have the handle which the client can
	// use when it goes through the name space searching
	// for its server.
	*ImportContext = (RPC_NS_HANDLE) local_ns_handle;

	// Open the gate. Let the worms in.
	--BindingLock;
	return status;
}

//
// Function:    FakeRpcNsBindingImportNext
// Return:      0 on success and an RPC error status on failure
// Description: The routine is called by a DCE application 
//					to retrieve a binding handle of a compatible server 
//					if found from the name space.
//
RPC_STATUS RPC_ENTRY
FakeRpcNsBindingImportNext(RPC_NS_HANDLE ImportContext,
			   RPC_BINDING_HANDLE * binding_handle)
{
	unsigned long local_ns_handle = (unsigned long) ImportContext;
	error_status_t status;
	unsigned char binding[STRMAX]; // contains string rep. of the 
				       // binding handle. 
	long length = sizeof(binding);

	// Since multiple threads have access to the name binding handle 
	// object we want to make sure the current thread is the sole
	// user of this binding object.
 
	++BindingLock;	
	
	// Make the actual RPC call to the proxy server
	status = RpcNsBindingImportNext_(
					  *nameBinding,
					 local_ns_handle,
					 &length,
					 binding);

	if (!CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcNsBindingImportNext_",
		  __LINE__, __FILE__)) {
		--BindingLock;
		return status;
	}
	
	status = RpcBindingFromStringBinding(binding,
					     binding_handle);
	
	CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcBindingFromStringBinding",
		  __LINE__, __FILE__);

	// Open the gate. Let the insects in.
	--BindingLock;
	return status;
	 
}

//
// Function:    FakeRpcNsBindingImportDone
// Return:      0 on success and an RPC error status on failure
// Description: Deletes the import context for searching the name
//					service. The call will go to the proxy server since
//					the import context data exists in the proxy server.
//
RPC_STATUS RPC_ENTRY
FakeRpcNsBindingImportDone(RPC_NS_HANDLE * ImportContext)
{
	unsigned long local_ns_handle = (unsigned long) *ImportContext;
	error_status_t status;

	// Since multiple threads have access to the name binding handle 
	// object we want to make sure the current thread is the sole
	// user of this binding object. 
	
	++BindingLock;

	// The thread is done using name binding object. 
	threadCount--;
	status = RpcNsBindingImportDone_(
					  *nameBinding,
						local_ns_handle);
	if (!CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcNsBindingImportDone_",
		  __LINE__, __FILE__)) {
		--BindingLock;
		return status;
	}

	//
	// A thread can delete this name binding handle object
	// only if it exists and no other thread is using it.
	//
	if (nameBinding && (threadCount == 0)) {
		delete nameBinding;
		nameBinding = (Binding *)NULL;
	}

	// Open the gate. Let the insects in.
	--BindingLock;
	return status;
}

//
// Function:    FakeRpcMgmtStopServerListening
// Return:      0 on success and an RPC error status on failure
// Description: This function tells the server to stop listening
//					for remote procedure calls.
//
RPC_STATUS RPC_ENTRY
FakeRpcMgmtStopServerListening(RPC_BINDING_HANDLE handle)
{

	error_status_t status, status1;
	unsigned char *stringBinding;
	Boolean previouslyConnected = false;

	// Since multiple threads have access to the name binding handle 
	// object we want to make sure the current thread is the sole
	// user of this binding object. 

	++BindingLock;
	status = RpcBindingToStringBinding(handle,
					&stringBinding);

	CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcBindingToStringBinding",
		  __LINE__, __FILE__);

	if (status != RPC_S_OK) {
		--BindingLock;
		return status;
	}
	//      
	// Only if it is not NULL do we have to connect to the
	// proxy server. Otherwise we might sitll have the
	// connection due to the name space lookup calls.
	//
	if (!nameBinding)
	   nameBinding = new Binding(ProxyServerName, ProxyServerPort, 
										(LoginContext*)0, RPCClient(Proxy,0,0));
	else
	   previouslyConnected = true;

	status1 = RpcMgmtStopServerListening_(
						*nameBinding,
						stringBinding);

	CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcMgmtStopServerListening_",
		  __LINE__, __FILE__);

	status =  RpcStringFree(&stringBinding);

	if (status != RPC_S_OK)
	{
		--BindingLock;
		return status;
	}

	if (!previouslyConnected) {
		delete nameBinding; 
		nameBinding = (Binding *)NULL;
	}

	--BindingLock;
	return status1;
}       

//
// Function:    FakeRpcMgmtIsServerListening
// Return:      0 on success and an RPC error status on failure
// Description: Tells whether a server is listening for remote
//					procedure calls or not.
//
RPC_STATUS RPC_ENTRY
FakeRpcMgmtIsServerListening(RPC_BINDING_HANDLE handle)
{
	
	error_status_t status, status1;
	unsigned char *stringBinding;
	Boolean previouslyConnected = false;

	// Since multiple threads have access to the name binding handle 
	// object we want to make sure the current thread is the sole
	// user of this binding object. 

	++BindingLock;
	status = RpcBindingToStringBinding(handle,
					&stringBinding);
	if (status != RPC_S_OK) {
		--BindingLock;
		return status;
	}
	//      
	// Only if it is not NULL do we have to connect to the
	// proxy server. Otherwise we might still have the
	// connection due to some name space lookup calls.
	//
	if (!nameBinding) {
	   nameBinding = new Binding(ProxyServerName, ProxyServerPort, 
										(LoginContext*)0, RPCClient(Proxy,0,0));
		if ((nameBinding->IsValid) == False) {
			status = RPC_S_NAME_SERVICE_UNAVAILABLE;
			CheckStatus(status, true, ErrorLevels::FatalMsg,
				   "Failed Binding to ProxyServer",
				   __LINE__, __FILE__);
			--BindingLock;
			return status;
		}
	}
	else
	   previouslyConnected = true;

	// Make the rpc call 
	status1 = RpcMgmtIsServerListening_(
					  *nameBinding,
					  stringBinding);

	if (!CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "RpcMgmtIsServerListening_",
		  __LINE__, __FILE__)) {
		--BindingLock;
	   return false;
	}

	status =  RpcStringFree(&stringBinding);

	if (status != RPC_S_OK) {
		--BindingLock;
		return status;
	}

	if (!previouslyConnected) {
		delete nameBinding; 
		nameBinding = (Binding *)NULL;
	}
	--BindingLock;
	return true;

}

//
// Function:    dce_error_inq_text
// Return:      return nothing
// Description: This function fills in the message text for a 
//					component status code used by client, server or
//					managment applications. 
//
void dce_error_inq_text(unsigned long status, 
			unsigned char *error_text,
			int *status1)
{
	switch (status) {
		case RPC_S_OK:
			strcpy((char *)error_text, "Success");
			break;
		case RPC_S_INVALID_BINDING:
			strcpy((char *)error_text, "Invalid binding handle");
			break;
		case RPC_S_WRONG_KIND_OF_BINDING:
			strcpy((char *)error_text,"Wrong kind of binding for operation");
			break;
		case RPC_S_INVALID_STRING_UUID:
			strcpy((char *)error_text, "String representation of the UUID not valid");
			break;
		case RPC_S_NO_MORE_BINDINGS:
			strcpy((char *)error_text, "No more bindings");
			break;
		case RPC_S_NAME_SERVICE_UNAVAILABLE:
			strcpy((char *)error_text, "Name service unavailable");
			break;
		case RPC_S_SERVER_UNAVAILABLE:
			strcpy((char *)error_text, "RPC server is unavilable: Either the proxyServer or the application server is down.");
			break;
		case RPC_S_UNKNOWN_AUTHN_SERVICE:
			strcpy((char *) error_text, "Unknown authentication service");
			break;
		case RPC_S_UUID_NO_ADDRESS:
			strcpy((char *) error_text, "Cannot get Ethernet or token-ring hardware address for this computer");
			break;
		case RPC_S_OUT_OF_MEMORY:
			strcpy((char *) error_text, "Rpc run-time library out of memory");
			break;
		case RPC_S_INVALID_TIMEOUT:   
			strcpy((char *) error_text, "Invalid timeout value");
			break;
		case EPT_S_NOT_REGISTERED:
			strcpy((char *) error_text, 
				"No more endpoints available from the EndPoint Mapper: Check for possible Interface uuid mismatch\n");
			break;
		case rpc_s_binding_incomplete:
			strcpy((char *) error_text, 
				"ProxyServer: Binding incomplete (no object ID and no end point)");
			break;
		case rpc_s_cant_getpeername:
			strcpy((char *) error_text, 
				"ProxyServer: Cannot get peername");
			break;
		case rpc_s_connection_closed:
			strcpy((char *) error_text, 
				"ProxyServer: No connection available");
			break;
		case rpc_s_class_version_mismatch:
			strcpy((char *) error_text, 
				"ProxyServer: RPC class version mismatch");
			break;
		case rpc_s_comm_failure:
			strcpy((char *) error_text, 
				"ProxyServer: Communication failure");
			break;
		case rpc_s_entry_not_found:
			strcpy((char *) error_text, 
				"ProxyServer: Name service entry not found");
			break;
		case rpc_s_incomplete_name:
			strcpy((char *) error_text, 
				"ProxyServer: Incomplete name");
			break;
		case rpc_s_invalid_arg:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid argument");
			break;
		case rpc_s_invalid_binding:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid binding");
			break;
		case rpc_s_invalid_endpoint:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid endpoint");
			break;
		case rpc_s_invalid_ns_handle:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid name server handle");
			break;
		case rpc_s_invalid_name_syntax:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid name syntax");
			break;
		case rpc_s_invalid_object:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid object entry");
			break;
		case rpc_s_invalid_rpc_protseq:
			strcpy((char *) error_text, 
				"ProxyServer: Invalid protocol sequence");
			break;
		case rpc_s_name_service_unavailable:
			strcpy((char *) error_text, 
				"ProxyServer: Name service not available");
			break;
		case rpc_s_protseq_not_supported:
			strcpy((char *) error_text, 
				"ProxyServer: Current protocol sequence not supported");
			break;
		case rpc_s_no_env_setup:
			strcpy((char *) error_text, 
				"ProxyServer: Environment variable not setup");
			break;
		case rpc_s_no_more_bindings:
			strcpy((char *) error_text, 
				"ProxyServer: No more bindings");
			break;
		case rpc_s_no_ns_permission:
			strcpy((char *) error_text, 
				"ProxyServer: No permission for name serivce operation");
			break;
		case rpc_s_not_rpc_entry:
			strcpy((char *) error_text, 
				"ProxyServer: Not an RPC entry");
			break;
		case rpc_s_wrong_kind_of_binding:
			strcpy((char *) error_text, 
				"ProxyServer: Wrong kind of binding of operation");
			break;
		case rpc_s_unknown_if:
			strcpy((char *) error_text, 
				"ProxyServer: Unknown interface");
			break;
		case rpc_s_unsupported_name_syntax:
			strcpy((char *) error_text, 
				"ProxyServer: Unsupported name syntax");
			break;
		default:
			sprintf((char *)error_text, "Error return code of %d ",
				status);        
			break;
		}
	*status1 = RPC_S_OK;
}


//
// Function:    uuid_compare
// Return:     -1 if first param precedes the second param in order.
//              0 if first param is equal to the second param in order.
//					 1 if first param follows the second param in order.
//
// Description:  Compares two UUIDs and determines their order.
//
signed32 uuid_compare(uuid_t *uuid1,
		      uuid_t *uuid2,
		      unsigned32 *status)
{
	int ret;
	error_status_t lstatus;

	unsigned char * uuid1_str, * uuid2_str;

	// both are NULL
	if ((!uuid1) && (!uuid2))
	   return 0;
	// only if uuid1 is NULL
	if (!uuid1)
	   return -1;
	if (!uuid2)
	   return 1;

	// Convert them into strings since 
	// both uuids are of opaque types.

	uuid_to_string(uuid1, &uuid1_str, &lstatus);
	uuid_to_string(uuid2, &uuid2_str, &lstatus);

	ret = strcmp((char *)uuid1_str, (char *)uuid2_str);

        rpc_string_free(&uuid1_str, &lstatus);
	rpc_string_free(&uuid2_str, &lstatus);
	*status = (unsigned32) lstatus; 
	return ret;

}

//
// Function:    uuid_equal
// Return:      0 on success and -1 on failure (not equal).
// Description:  Compares two UUIDs and determines if they are equal.
//
Boolean uuid_equal(uuid_t *uuid1, 
		   uuid_t *uuid2, 
		   unsigned32 *status)
{
	int ret;
	error_status_t lstatus;
	unsigned char * uuid1_str, * uuid2_str;

	uuid_to_string(uuid1, &uuid1_str, &lstatus);
	uuid_to_string(uuid2, &uuid2_str, &lstatus);
	ret = strcmp((char *)uuid1_str, (char *)uuid2_str);
	rpc_string_free(&uuid1_str, &lstatus);
	rpc_string_free(&uuid2_str, &lstatus);
	*status = lstatus;
	if (!ret)
	   return true;
	else
	   return false;
}

#ifdef _WINDOWS

Boolean ObjInitInstance(HINSTANCE);

int      alreadyInitialized = FALSE;
WNDCLASS objwc;
HANDLE   globalInstance;
HWND     hWnd;

int FAR PASCAL LibMain(HANDLE hInstance,
		       WORD  wDataSeg,
		       WORD  cbHeapSize,
		       LPSTR lpszCmdLine)
{
        objwc.style = NULL;                    // Class style(s).                    
        objwc.lpfnWndProc = DefWindowProc;// Function to retrieve messages for  
        objwc.cbClsExtra = 0;             // No per-class extra data.           
        objwc.cbWndExtra = 0;             // No per-window extra data.          
        objwc.hInstance = hInstance;      // Application that owns the class.   
        objwc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        objwc.hCursor = NULL;
        objwc.hbrBackground = NULL;
        objwc.lpszMenuName =  NULL;       // Name of menu resource in .RC file. 
        objwc.lpszClassName = "Objdce";   // Name used in call to CreateWindow. 
	     globalInstance = hInstance;

        return RegisterClass(&objwc);
}

int FAR PASCAL _export WEP (int nParameter)
{
	return(1);
}
#endif			


void ObjtranFirstInit()
{

#ifdef _WINDOWS
    if (alreadyInitialized == FALSE) {
		if (!ObjInitInstance(globalInstance))
		{
		    MessageBox(NULL, "ERROR: failed to initialize application.",
   	                  "InitApplication", MB_OK);
		    return;
		}

      RpcWinSetYieldInfo(hWnd, FALSE, 1, 0);
		alreadyInitialized = TRUE;

    }
#endif
    return; 
} 

Boolean ObjInitInstance(HINSTANCE hInstance)
{
#ifdef _WINDOWS
	/* Create a main window for this application instance.  */

	hWnd = CreateWindow(
		"Objdce",                        // See RegisterClass() call.          
		"Objdce Client Library",         // Text for window title bar.
		WS_OVERLAPPEDWINDOW,
		0,              // Default horizontal position.       
		0,              // Default vertical position.         
		200,            // Default width.                     
		100,            // Default height.                    
		NULL,           // Overlapped windows have no parent. 
		NULL,           // Use the window class menu.         
		hInstance,      // This instance owns this window.    
		NULL            // Pointer not needed.                
		);

	if (!hWnd)
		return (FALSE);
#endif

	return (TRUE);        // Returns the value from PostQuitMessage 
}

//
// Function: pthread_create
// Return  : -1 on failure and 0 on success
//
// Description: This function is a wrapper around the Windows NT
//              CreatThread function. The argument mapping is the
//              same except for the second arg that is passed to
//              pthread_create.
//              
//              
int pthread_create(pthread_t 	*threadHandle,
						 pthread_attr_t   threadAttr,
						 pthread_startroutine_t ThreadFunc,
						 pthread_addr_t argument)
{

	DWORD IdThread;
	HANDLE LocalHandle;
   DWORD stack = 0;

	if (threadAttr.attribute != 0)
		stack = threadAttr.attribute;

	LocalHandle=CreateThread(NULL,    	// no security attribute
								    stack,    	// stack size normal if 0
									 (LPTHREAD_START_ROUTINE) ThreadFunc, // starting
																					  // function
									 argument,  // parameter to ThreadFunc
									 THREAD_PRIORITY_NORMAL,
									 &IdThread);

   if (LocalHandle == NULL) {
		LOGERR << "CreateThread failed!!\n";
		return (-1);
	}		
   *threadHandle = LocalHandle;
	return (0);

}

//
// Function: pthread_cancel
// Return: -1 on failure and 0 on success
// Description: It sends a cancel request to a specified
//					 thread. It is basically a terminating signal
//  				 to the thread. The thread is identified by its
//					 object handle.
//
int pthread_cancel(pthread_t thread)
{
	BOOL  ret;


	ret = TerminateThread(thread, 0);
	if (ret == FALSE) {
		LOGERR << "TerminateThread failed!!\n";
		return -1;   
   } 
	return 0;
}

//
// Function: pthread_getprio
// Return: -1 on failure or the priority value on success
// Description: This function obtains the current 
//					 priority of a thread. The current 
//					 priority is different from the initial
//					 priority of if the pthread_setprio()
//					 routine is called.
//
int pthread_getprio(pthread_t thread)
{
	return (GetThreadPriority(thread));
}

//
// Function: pthread_setprio
// Return: -1 on failure or previous priority on success.
// Description: New priority value of the thread specified
//					 in thread. The priority value depends on
//					 scheduling policy. 
//
int pthread_setprio(pthread_t thread, int priority)
{
	BOOL ret;
	int prev = GetThreadPriority(thread);

	ret = SetThreadPriority(thread, priority);

	if (ret == FALSE)
		return(-1);

	return (prev);
}

//
// Function: pthread_join
// Return: 0 on succes or -1 on failure
// Description: This routine awaits the specified thread's 
//					 termination. The synchronization mechanism 
//					 is prossible via an event object. The object 
//					 in our case is referring to the thread whose  
//					 termination we are so eagerly awaiting. If the
//					 thread terminates (thus going into a signaled
//					 state) the WaitForSingleObject routine returns.  
//					 
//						 
// Note:: In the future release we have
// to protect this variable.
unsigned long exitCode;
 
int pthread_join(pthread_t thread, pthread_addr_t *status)
{
	 
	 //
    // Wait for the specified thread to go
    // into a signaled state before we return.
    // INFINITE indicates wait indefinitely
	 // if the specified thread has not terminated. 
	 //
	 if (WaitForSingleObject(thread, INFINITE) == WAIT_FAILED)  {
		 LOGERR <<	"pthread_join: WaitForSingleObject failed with error # " 
				  << GetLastError() << "\n"; 
		 return (-1);
	 }
	 else {
		 // Get the threads exit code which has been set by
		 // TerminateThread call. Should put a lock around
		 // the exit code variable!!

	    GetExitCodeThread(thread, &exitCode); 
		 *status = &exitCode;
		 return (0);
	}
}


//
// Function:  pthread_detach
// Return:    0 on success and -1 on failure
// Description: This function is called when a thread's
//					 resources are to be reclaimed once it 
//				    terminates. If this routine is called 
//					 while the thread is still alive, the kernel
//					 will not terminate the thread by taking away
//					 its resources. The pthread_detach is a marker
//					 to the object manager that only when the thread
//				    terminates should its resources be taken away. 
//					 Ofcourse, if the thread is dead, its resources will
//				    be up for grabs by the system. 
 
int pthread_detach(pthread_t thread)
{
	if (CloseHandle(thread) == FALSE)
		return -1;
	else
		return 0;

}

// 
// Function: pthread_yield
// Return:  void
// Description: Notify the scheduler that the current thread
//					 is willing to release the processor.	
//
void pthread_yield()
{
	if (SuspendThread(GetCurrentThread()) == -1) 
		LOGERR << "SuspendThread failed!!\n";

}

// Function: pthread_self
// Return: An identifier to the current thread
// Description: Returns a NT thread id

// Note:: In the future release we have to protect this 
// variable.

DWORD ThreadId;
pthread_addr_t pthread_self()
{
   ThreadId = GetCurrentThreadId();	
	return ((pthread_addr_t)&ThreadId);

}
// Function: pthread_exit
// Return:   void
// Description: Terminates the calling thread.
//
void pthread_exit(pthread_addr_t  status)
{
	DWORD *pstatus = (DWORD *) status;
	ExitThread(*pstatus);
}

//
// Function: pthread_attr_setstacksize
// Return: void
// Description: Changes the stacksize attribute of thread
//					 creation. Under Posix only the main or initial
//				    thread can dynamically extend its stack.

int pthread_attr_setstacksize(pthread_attr_t *attr, long stacksize)
{
	attr->attribute = stacksize;	
	return 0;
}

//
// Function: pthread_attr_create
// Return:   void
// Description: Thread attributes are created by this routine
//					 and stored in a handle. 

int pthread_attr_create(pthread_attr_t *attr)
{
	attr->attribute = 0; // Create the default stack.
	return 0;
}

int pthread_setcancel(int state)      { return 0;}
int pthread_setasynccancel(int state) { return 0;}
int pthread_testcancel()    { return 0;}
 
// Function: pthread_cond_int
// Return:   0 on success and -1 on failure
// Description: This function creates an event object. In our
//	        case this event object is a condition variable.
//		 
int pthread_cond_init(pthread_cond_t *condRep, pthread_condattr_t attr)
{
	HANDLE objectHandle;

	objectHandle =  CreateEvent(NULL,  // No security attributes
				    				    FALSE, // automatically reset to
					   						  // to non-signaled state after
					   						  // a signal is received 
				    					 FALSE, // initial state is 
					   					    // non-signaled.
				    					 "Posix Cond Var"); // Name of event 

	if (objectHandle == NULL) {
		LOGERR << "Create Event failed!!\n";
		return -1; 
	}
	*condRep = objectHandle;
	return 0;
}


//
// Function: pthread_attr_delete
// Return: 0 on success
// Description: This function deletes the attribute 
//					 value. The only attribute value we
//					 use now is the stacksize.
//
int pthread_attr_delete(pthread_attr_t *attr)
{
	attr->attribute = 0;
	return 0;
}

//
// Function: pthread_cond_destroy
// Return:   0 on success and -1 on failure
// Description: This function deletes the condition 
//					 variable object. 
//
int pthread_cond_destroy(pthread_cond_t *cond)
{
	//
   // Waring: Results are unpredictable if the condition
   // variable is destroyed while threads are waiting for 
	// a signal.    
   //
	if (CloseHandle(*cond) == FALSE) 
		 return(-1); 
   return(0);

} 

// 
// Function: pthread_cond_wait
// Return: 0 on success and -1 on failure
// Description: Causes the calling thread to wait for a condition 
//					variable to be signaled or broadcast. This function 
//					should only be called after a mutex has been succesfully
//					locked by the calling thread.
				      
int pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
{     
		
		// The mutex has to be released before it goes
		// into a wait state to give other threads waiting
		// on this mutex to be woken up. If we don't release 
	        // the mutex we will cause a deadlock since the other
		// threads will never wake up and the current thread will
		// go into a wait state forever.

		if (ReleaseMutex(mutex->handle) == FALSE) {
			LOGERR << "ReleaseMutex failed!!\n";
		   return (-1);
		}

		
		// Go into a wait state. Wait for a condition signal 
		// from another thread (pthread_cond_signal or 
		// or pthread_cond_broadcast).

		// **Important Note **
                // In case another thread calls pthread_cond_signal
		// before the current thread waits for a condition variable.
		// we won't lose the signal and sleep for ever. 
		// The condtion variable will still be signaled if there 
		// are no other threads waiting on the condition variable.
		// The above only holds true if it is pthread_cond_signal. 
		// If it is a pthread_cond_broadcast then we do LOSE 
		// the signal and sleep forever since the semantics of 
		// pthread_cond_broadcast on NT forces us to lose the signal 
		// if there are no threads in a wait state.	

		if (WaitForSingleObject(*cond, INFINITE) == FALSE) {
			LOGERR << "WaitForSingleObject failed\n"; 
			return(-1);
		}
		
		// Try acquiring the mutex lock again. If we aren't
		// successful in acquiring the lock just wait for 
		// another thread to release the mutex lock. Just
		// wait for a ReleaseMutex signal. Again, if another
		// thread releases the mutex lock before we wait
		// on mutex object we would still receive the signal.

		if (WaitForSingleObject(mutex->handle, INFINITE) == FALSE) {
			LOGERR << "WaitForSingleObject failed\n";
			return(-1);
		}
		return(0);
}

// 
// Function: pthread_cond_timedwait
// Return: 0 on success and -1 on failure
// Description: Cause the calling thread to wait for a condtion
//					 variable to be signaled or broadcast.

int pthread_cond_timedwait(pthread_cond_t *cond,
									pthread_mutex_t *mutex,
									timespec_t *abstime)
{
		
		if (WaitForSingleObject(*cond, abstime->tv_sec * 1000) == FALSE) {
			LOGERR << "WaitForSingleObject failed\n";
			return(-1);
		}
		return(0);

}
	
//
// Function: pthread_cond_signal
// Return: 0 on failure and -1 on failure
// Description: Wakes one thread up that is waiting on a
//					condition variable.

int pthread_cond_signal(pthread_cond_t *cond)
{
	 if (SetEvent(*cond) == FALSE) 
		 return(-1);
	 else
		 return(0);
}
//
// Function: pthread_cond_broadcast
// Return: 0 on failure and -1 on failure
// Description: Wakes all threads that are waiting on a 
//					condition variable. 

int pthread_cond_broadcast(pthread_cond_t *cond)
{
	 // 
	 // Right now PulseEvent only works 
	 // if there are threads waiting on the
	 // condition variable. If none of the 
	 // threads are waiting we lose the signal.
	 // Thus any subsequent thread that does a wait
	 // will never get a signal. Use this call
	 // carefully.
	 //
	 if (PulseEvent(*cond) == FALSE)
		 return(-1);
	 else
		 return(0);
}

//
// Function: pthread_mutex_init
// Return:   0 on success and -1 on failure 
// Description: Create a mutex variable which is
//					 initialized as unlocked. 
// 
int pthread_mutex_init(pthread_mutex_t *mutex,
							  pthread_mutexattr_t attr)
{
	mutex->attribute = attr.attribute;

	if (attr.attribute == MUTEX_RECURSIVE_NP)
	{
		mutex->handle  = CreateMutex(NULL,   // no security attributes
										     FALSE,  // the calling thread does not own
													    // the mutex variable
   										  "Mutex");
	}
	else {
		mutex->handle = CreateSemaphore(NULL, // no security attributes
											 	  1,    // set it to a signaled state
											     1,    // maximum semaphore count
												 "SemOp");
	}
 
	if (mutex->handle == NULL) {
		LOGERR << "CreateMutex failed to initialize!!";
		return (-1);
	}
	return(0);
}

//
// Function: pthread_mutex_lock
// Return:	 0 on success or -1 on failure
//
// Description: The function locks a mutex.  If the mutex
//					 is locked when a thread calls this routine,
//					 the thread waits for the mutex to be unlocked.
//					 Otherwise, the thread becomes the owner. 
//					 However, things are a little more compilcated than
//					 the above scenario.
//					
//         	    1. Fast Locks: According to Posix specification
//				    if a thread relocks its own mutex it should be 
//					 deadlocked. We achieve this behavior by using a binary
//					 semaphore.   				 
//
//					 2. Recursive Locks: According to Posix specification
//					 a thread should be able to relock a mutex as many times
//					 as possible. We achieve this behavior by using 
//					 a mutex object.
// 
//					 3. Non-Recursive Locks: According to Posix specification 
//					 a thread trying to relock its own mutex should result
//					 in an error and the thread returning immediately. 
//					 We achieve the behavior by calling pthread_mutex_trylock
//					 routine and using a binary semaphore operation.
//					    

int pthread_mutex_lock(pthread_mutex_t *mutex) 
{
	
	DWORD ret;

	//
	// Check what type of mutex lock we are dealing with.
	//
	if (mutex->attribute == MUTEX_NONRECURSIVE_NP)
	{
			//
			// Now we have to emulate a non-recursive lock
			// Any thread that currently does not
			// own the mutex object and if it trys to acquire 
			// a locked mutex will fail and return immediately.
			// The thread won't block.
			// 
			ret = pthread_mutex_trylock(mutex);
			if ((ret == -1) || (ret == 0))
				return (-1);
			else
				return (0);
	}
	//
	// Block indefinitely if mutex lock not available
	//
	if ((ret = WaitForSingleObject(mutex->handle, INFINITE)) == WAIT_FAILED) {
		LOGERR << "pthread_mutex_lock: WaitForSingleObject failed on a mutex object";
		LOGERR << " Error number = " << GetLastError() << "\n"; 
		return(-1);
	}
	//
	// Just because the call succeeded doesn't mean
	// that the thread has possession of the mutex lock.
	// It is possible that the previous thread which
	// owned the mutex died without releasing ownership
	// of the mutex.
	//      
	
	if (ret == WAIT_ABANDONED) 
		return (-1);
	else 
		// Acquired the mutex lock 
		return (0);

}

//
// Function: pthread_mutex_unlock
// Return: 0 on success
// Description: Relase the mutex variable thus
//					 generating a signal to any waiting
//					 threads.
// 				

int pthread_mutex_unlock(pthread_mutex_t *mutex)
{
	long prev;

	if (mutex->attribute == MUTEX_RECURSIVE_NP) 
	{
		if (ReleaseMutex(mutex->handle) == FALSE)
			return (-1);
		else
			return (0);
	}
	else
	{
		if (ReleaseSemaphore(mutex->handle, 1, &prev) == FALSE)
			return (-1);
		else
			return (0);
	}
}


//
// Function: pthread_mutex_trylock
// Return:   0 on successfully locking the mutex
//				 1 on not being able to lock the mutex 
//				-1 on faiure   
//
// Description: This routine allows the thread to quickly
//					check if the mutex is available for
//					acqusition. If the thread cannot acquire the
//					mutex, it returns immediately without 
//					blocking. 
// 

int pthread_mutex_trylock(pthread_mutex_t *mutex)
{

	DWORD ret;	
	const int waitPeriod = 10; // In milliseconds

	//
	// We will keep 10 milliseconds as the maxmimum time
	// the thread should wait before returning. 
	//

	if ((ret = WaitForSingleObject(mutex->handle, waitPeriod)) == WAIT_FAILED) {
		LOGERR  << "pthread_mutex_trylock: WaitForSingleObject failed,";
		LOGERR  << " Error number = " << GetLastError() << "\n";
		return(-1);
	}
	// 
	// Just because the call succeeded doesn't
	// necessarily mean the thread has ownership
	// of the mutex lock. It is quite possible that  
	// the thread timed out (not acquiring the mutex)
	// or the thread that previously had the ownership
	// of the mutex died without releasing it. 
	//
	if (ret == WAIT_TIMEOUT)
		return (0); // Could not lock the mutex  
	else
	   if (ret == WAIT_ABANDONED)
			return (-1); // Thread that owned the mutex terminated  
							 // without releasing it
		else
			return (1);	 // Acquired the mutex
}

//  
//	 Function: pthread_mutex_destroy
//  Return:   0 on success
//  Description: Destory the mutex object
//
int pthread_mutex_destroy(pthread_mutex_t * mutex)
{
	mutex->attribute = 0;
	CloseHandle(mutex->handle);
	return (0);
}


//
//  Function: pthread_mutexattr_setkind_np
//	 Return: 0 on success
//  Description: This routine sets a global variable 
//					  to the type of lock mechanism that is
//					  associated with the mutex
//
int pthread_mutexattr_setkind_np(pthread_mutexattr_t *mutex, int kind)
{
	// This global variable is used by us to determine
	// if we have to emulate a non-recurisve
	// mutex lock when pthread_mutex_lock is called.
	
	mutex->attribute = kind;
	return 0;
}

//
// Function: pthread_mutexattr_create
// Return: 0 on success
// Description: Create a mutex attribute object.
//		

int pthread_mutexattr_create(pthread_mutexattr_t *mutex)
{
	  //
	  // Set the default attribute to 
	  // fast locks.
	  //
	  mutex->attribute = MUTEX_FAST_NP; 
	  return(0);
}
	
//
// Function: pthread_mutexattr_delete
// Return: 0 on success
// Description: Deletes a mutex attribute object.
//		
 
int pthread_mutexattr_delete(pthread_mutexattr_t *mutex)
{
	 //
	 // Delete its attribute 
	 //
	 mutex->attribute = 0;
	 return(0);
}

