/***********************************************************************
 * Copyright (C) 1994 Cambridge Technology Partners, all rights reserverd 
 *
 * Module: rpcmap.h
 * 
 * Description: This module serves the purpose of providing extensions to
 *              the MicroSoft (MS) RPC toolkit. The current release of MS
 *	        RPC toolkit does not provide the functionality to access the
 *		DCE namespace. This module provides prototypes for proxy
 *		routines that access the DCE name space. The module contains
 *		the declarations of Posix pthread typedefs. Last but not least
 *		it provides an wrapper class for objdce logging.
 *
 * Created:     Rizwan Mallal, Jan 10th 1994
 * 
 *
 *************************************************************************/

#ifndef RPCMAP_H
#define RPCMAP_H

extern "C" {
#include <rpc.h>
}

#include <ctplog.h>
#include <ctplock.h>
#include <objdce/defs.H>
#include <objdce/errorLevels.H>
#include <stdarg.h>



//
// Below are the two attributes needed by the proxy routines
// to connect to the proxServer on the Unix side.
//
#define ProxyServerName "dino"
#define ProxyServerPort  2400

//
// Constants for buffer strings.
//
#define MAXBUF 1024
#define MINBUF 20
#define STRMAX	81



//
//
// LogReport is a version of logging class to 
// provide access to CTP's logging facility
// while keeping the same logging interface 
// as Citibanks.
//
class LogReport {
	public:
		void Report(ErrorLevels::ReportingLevels, 
			    const char *);
		void ReportMsg(ErrorLevels::ReportingLevels, 
				const int);
		void VReport(ErrorLevels::ReportingLevels,
			     const char *, 
				...);
		void VReportMsg(ErrorLevels::ReportingLevels,
				const int,
				...);
};
extern LogReport ObjtranReport;


//
// Method: LogReport:Report
// Return: void
// Description: This method prints out a single string. Ignore the level
//              that is passed out since the information should only get 
//              when log level is set to LOG_INFORMATIONAL by an application.
// 
inline void LogReport::Report(ErrorLevels::ReportingLevels level,
			 const char * msg)
{
	LOGDEBUGLEVEL(CTPLog::MSG_INFORMATIONAL) << msg << "\n"; 
}


//
// Method: LogReport:ReportMsg
// Return: void
// Description: This method prints out the msg number indicating what
//              what kind of RPC call was made. Again we ignore the level 
//              since we will depend on log level being set to 
//              LOG_INFORMATIONAL by a calling application.
//

inline void LogReport::ReportMsg(ErrorLevels::ReportingLevels level,
       			  const int msgno)
{
	LOGDEBUGLEVEL(CTPLog::MSG_INFORMATIONAL) << "Msg No: " << msgno << "\n";

}


//
// Method: LogReport:ReportV
// Return: void
// Description:  This method prints a single message where
//               msg contains the format information.
//               The variable arguments have the actual parameters.
//

inline void LogReport::VReport(ErrorLevels::ReportingLevels, const char * msg,
			  ...)
{
	char tempString[MAXBUF];
	int ret;

	va_list ap;
	va_start(ap, msg);
	ret = vsprintf(tempString, msg, ap);
	va_end(ap);
	LOGDEBUGLEVEL(CTPLog::MSG_INFORMATIONAL) << tempString << "\n"; 

}

//
// Method: LogReport:VReportMsg
// Return: void
// Description: This method takes a msg number and variable number of arguments
//              to spit out a single informational message. Again we ignore 
//              the level number and depend on LOG_INFORMATIONAL being set
//              in the calling application's code.
//

inline void LogReport::VReportMsg(ErrorLevels::ReportingLevels, const int msg,
			  ...)
{
	int ret;
	char bufString[MINBUF];
	char tempString[MAXBUF];

	va_list ap;
	va_start(ap, msg);
	sprintf(bufString, "Msg no: %d", msg);  // convert an int to its ascii rep.

	ret = vsprintf(tempString, bufString, ap); 
	va_end(ap);
 	LOGDEBUGLEVEL(CTPLog::MSG_INFORMATIONAL) << tempString << "\n";	
}



//******************************************************************
//
// Below is the listing of all the function prototypes and typedefs
// that we have used to provide a workaround for those DCE functions 
// that are not supported as the part of the MicroSoft's RPC API 1.0
//
//******************************************************************

#define CANCEL_OFF	0
#define CANCEL_ON		1

#define MUTEX_FAST_NP			0
#define MUTEX_RECURSIVE_NP 	1
#define MUTEX_NONRECURSIVE_NP 2

typedef unsigned int unsigned32;
typedef int signed32;

signed32 uuid_compare(uuid_t *, uuid_t *, unsigned32*);
Boolean  uuid_equal  (uuid_t *, uuid_t *, unsigned32*);
void dce_error_inq_text(unsigned long, 	unsigned char *, int *);

// Current Posix threads aren't supported
// on NT but we will provide them on NT and 
// Windows 3.1. 

typedef struct {
	HANDLE handle;
	int    attribute;
} cma_t_handle;

//
// Time structure defined in DCE.
// Since it is not suppored by MS RPCs 
// we will support it on NT and Windows 3.1.
//

struct timespec {
	int tv_sec;
	int tv_nsec;
};

typedef cma_t_handle pthread_mutex_t;
typedef struct  timespec timespec_t;

typedef HANDLE 	pthread_t;
typedef HANDLE  	pthread_cond_t;
typedef cma_t_handle   pthread_condattr_t; 
typedef cma_t_handle   pthread_mutexattr_t;
typedef cma_t_handle   pthread_attr_t;

typedef void *		pthread_addr_t;
typedef void * (* pthread_startroutine_t) (void *);

extern pthread_condattr_t	 pthread_condattr_default;
extern pthread_attr_t     	 pthread_attr_default;


// Rename DCE handles to MicroSoft RPC handles
#define rpc_binding_handle_t 		 RPC_BINDING_HANDLE
#define rpc_if_handle_t			 RPC_IF_HANDLE
#define rpc_ns_import_handle_t           RPC_NS_HANDLE


#define dce_c_error_string_len 160       // length of returned string
typedef unsigned char dce_error_string_t[dce_c_error_string_len];

#ifndef _ERROR_STATUS_T_DEFINED
typedef unsigned long error_status_t;
#define _ERROR_STATUS_T_DEFINED
#endif


RPC_STATUS RPC_ENTRY
FakeRpcMgmtIsServerListening(RPC_BINDING_HANDLE);
			
RPC_STATUS RPC_ENTRY
FakeRpcNsBindingImportBegin(unsigned long,
			    unsigned char *,
		            RPC_IF_HANDLE,
			    UUID *,
			    RPC_NS_HANDLE *);

RPC_STATUS RPC_ENTRY
FakeRpcNsBindingImportNext(RPC_NS_HANDLE,
			   RPC_BINDING_HANDLE *);

RPC_STATUS RPC_ENTRY
FakeRpcNsBindingImportDone(RPC_NS_HANDLE *);

RPC_STATUS RPC_ENTRY
FakeRpcMgmtStopServerListening(RPC_BINDING_HANDLE);

int pthread_attr_create(pthread_attr_t *);
int pthread_attr_delete(pthread_attr_t *);
int pthread_attr_setstacksize(pthread_attr_t *, long);

int pthread_cancel(pthread_t);
int pthread_cond_destroy(pthread_cond_t *);
int pthread_cond_init(pthread_cond_t *, pthread_condattr_t);
int pthread_cond_broadcast(pthread_cond_t *);
int pthread_cond_signal(pthread_cond_t *);
int pthread_cond_timedwait(pthread_cond_t *, pthread_mutex_t *,
			   struct timespec *);

int pthread_cond_wait(pthread_cond_t *, pthread_mutex_t *);
int pthread_create(pthread_t *, pthread_attr_t,
		   pthread_startroutine_t, pthread_addr_t);

int pthread_detach(pthread_t);

void pthread_exit(pthread_addr_t);

int pthread_getprio(pthread_t);

int pthread_join(pthread_t, pthread_addr_t *);

int pthread_mutexattr_create(pthread_mutexattr_t *);
int pthread_mutexattr_delete(pthread_mutexattr_t *);

int pthread_mutex_destroy(pthread_mutex_t *);
int pthread_mutex_init(pthread_mutex_t *, pthread_mutexattr_t);
int pthread_mutexattr_setkind_np(pthread_mutexattr_t *, int);
int pthread_mutex_lock(pthread_mutex_t *);
int pthread_mutex_trylock(pthread_mutex_t *);
int pthread_mutex_unlock(pthread_mutex_t *);


int pthread_setasynccancel(int);
int pthread_setcancel(int);
int pthread_setprio(pthread_t, int);

int pthread_testcancel(void);
pthread_addr_t pthread_self();
void pthread_yield();



//****************************************************** 
//
// The following macros convert DCE standard RPC calling
// convention to one used by MicroSoft. The main 
// difference is the use of upper/lower alphabets used
// by MicroSoft for their RPC API functions. While 
// DCE uses all lower letters separated by underbars.
// Also, the status report variable is passed in as a
// parameter in DCE while MicroSoft returns a value
// to the status variable.
//
//*****************************************************

#define rpc_binding_copy(a1, a2, a3) \
	*a3 = RpcBindingCopy(a1, a2)

#define rpc_binding_free(a1, a2) \
	*a2 = RpcBindingFree(a1)

#define rpc_binding_from_string_binding(a1, a2, a3) \
	*a3 = RpcBindingFromStringBinding(a1, a2)

#define rpc_binding_inq_object(a1, a2, a3)\
	*a3 = RpcBindingInqObject(a1, a2)

#define rpc_binding_reset(a1, a2)\
	*a2 = RpcBindingReset(a1)

#define rpc_binding_set_object(a1, a2, a3) \
	*a3 = RpcBindingSetObject(a1, a2)

#define rpc_binding_to_string_binding(a1, a2, a3) \
	*a3 = RpcBindingToStringBinding(a1, a2)

#define rpc_string_binding_compose(a1, a2, a3, a4, a5, a6, a7) \
	*a7 = RpcStringBindingCompose(a1, a2, a3, a4, a5, a6) 

#define rpc_ep_resolve_binding(a1, a2, a3) \
	*a3 = RpcEpResolveBinding(a1, a2)

#define rpc_string_free(a1, a2)	\
	*a2 = RpcStringFree(a1)

#define rpc_string_binding_parse(a1, a2, a3, a4, a5, a6, a7) \
	*a7 = RpcStringBindingParse(a1, a2, a3, a4, a5, a6)

#define rpc_mgmt_is_server_listening(a1, a2) \
	*a2 = FakeRpcMgmtIsServerListening(a1) 

#define rpc_mgmt_set_com_timeout(a1, a2, a3)\
	*a3 = RpcMgmtSetComTimeout(a1, a2)

#define rpc_mgmt_inq_com_timeout(a1, a2, a3)\
	*a3 = RpcMgmtInqComTimeout(a1, a2)

#define rpc_ns_binding_import_begin(a1, a2, a3, a4, a5, a6) \
	*a6 = FakeRpcNsBindingImportBegin(a1, a2, a3, a4, a5) 

#define rpc_ns_binding_import_next(a1, a2, a3) \
	*a3 = FakeRpcNsBindingImportNext(a1, a2)

#define rpc_ns_binding_import_done(a1, a2) \
	*a2 = FakeRpcNsBindingImportDone(a1)

#define rpc_mgmt_stop_server_listening(a1, a2)\
	*a2 = FakeRpcMgmtStopServerListening(a1)

#define uuid_from_string(a1, a2, a3)\
	*a3 = UuidFromString(a1, a2)

#define uuid_to_string(a1, a2, a3)\
	*a3 = UuidToString(a1, a2)

#define uuid_create(a1, a2)\
	*a2 = UuidCreate(a1)


inline int pthread_delay_np(const timespec *sleep)
{
	//
	// Make the thread wait for so many milliseconds.
	//
	Sleep((sleep->tv_sec) * 1000);
	return(0);
}

 
// **************************************************
//
// Defines of those constants that are provided by
// the DCE API but not by the MicroSoft RPC API.
//
// **************************************************

#define error_status_ok	(0)
#define rpc_s_ok			 RPC_S_OK
#define rpc_s_binding_incomplete         (382312699)

#define rpc_s_cant_getpeername		 (382312539)
#define rpc_s_connection_closed		 (382312502) 
#define rpc_s_class_version_mismatch	 (382312708)
#define rpc_s_comm_failure 		 (382312470)

#define rpc_s_entry_not_found		 (382312608)

#define rpc_s_incomplete_name		 (382312596)

#define rpc_s_invalid_arg		 (382312547)
#define rpc_s_invalid_binding		 (382312477)
#define rpc_s_invalid_endpoint		 (382312526)
#define rpc_s_invalid_ns_handle		 (382312665)
#define rpc_s_invalid_name_syntax	 (382312598)
#define rpc_s_invalid_object		 (382312506)
#define rpc_s_invalid_rpc_protseq	 (382312480)

#define rpc_s_mgmt_op_disallowed	 (382312557)

#define rpc_s_name_service_unavailable	 (382312595)
#define rpc_s_protseq_not_supported	 (382312541)
#define rpc_s_no_env_setup		 (382312644)
#define rpc_s_no_more_bindings		 (382312629)
#define rpc_s_no_ns_permission		 (382312616)
#define rpc_s_not_rpc_entry		 (382312631)

#define rpc_s_wrong_kind_of_binding      (382312549)

#define rpc_s_unknown_if		 (382312492)
#define rpc_s_unsupported_name_syntax	 (382312614)

#define ept_s_not_registered		 (382312662) 


#define rpc_c_ns_syntax_dce		RPC_C_NS_SYNTAX_DCE
#define rpc_c_ns_syntax_default         RPC_C_NS_SYNTAX_DEFAULT
#define rpc_c_binding_infinite_timeout  RPC_C_BINDING_INFINITE_TIMEOUT
#define rpc_c_binding_min_timeout	RPC_C_BINDING_MIN_TIMEOUT
#define rpc_c_binding_default_timeout   RPC_C_BINDING_DEFAULT_TIMEOUT
#define rpc_c_binding_max_timeout	RPC_C_BINDING_MAX_TIMEOUT


#define NOT_IMPLEMENTED                 -32767

#define rpc_s_access_control_info_inv   RPC_S_INVALID_AUTH_IDENTITY   
#define rpc_s_auth_bad_integrity        NOT_IMPLEMENTED+1 
#define rpc_s_auth_badaddr              NOT_IMPLEMENTED+2              
#define rpc_s_auth_baddirection         NOT_IMPLEMENTED+3 
#define rpc_s_auth_badkeyver            NOT_IMPLEMENTED+4 
#define rpc_s_auth_badmatch             NOT_IMPLEMENTED+5 
#define rpc_s_auth_badorder             NOT_IMPLEMENTED+6  
#define rpc_s_auth_badseq               NOT_IMPLEMENTED+7
#define rpc_s_auth_badversion           NOT_IMPLEMENTED+8
#define rpc_s_auth_field_toolong        NOT_IMPLEMENTED+9
#define rpc_s_auth_inapp_cksum          NOT_IMPLEMENTED+10
#define rpc_s_auth_method               NOT_IMPLEMENTED+11
#define rpc_s_auth_modified             NOT_IMPLEMENTED+12
#define rpc_s_auth_msg_type             NOT_IMPLEMENTED+13 
#define rpc_s_auth_mut_fail             NOT_IMPLEMENTED+14
#define rpc_s_auth_nokey                NOT_IMPLEMENTED+15
#define rpc_s_auth_not_us               NOT_IMPLEMENTED+16  
#define rpc_s_auth_repeat               NOT_IMPLEMENTED+18
#define rpc_s_auth_skew                 NOT_IMPLEMENTED+19
#define rpc_s_auth_tkt_expired          NOT_IMPLEMENTED+20
#define rpc_s_auth_tkt_nyv              NOT_IMPLEMENTED+21
#define rpc_s_authn_authz_mismatch      RPC_S_UNKNOWN_AUTHN_TYPE
#define rpc_s_binding_has_no_auth       RPC_S_BINDING_HAS_NO_AUTH
#define rpc_s_invalid_checksum          NOT_IMPLEMENTED+22
#define rpc_s_invalid_crc               NOT_IMPLEMENTED+23
#define rpc_s_invalid_credentials       NOT_IMPLEMENTED+24
#define rpc_s_unknown_authn_service     RPC_S_UNKNOWN_AUTHN_SERVICE
#define rpc_s_unsupported_auth_subtype  NOT_IMPLEMENTED+25
#define rpc_s_unsupported_protect_level NOT_IMPLEMENTED+26

#define ept_s_server_unavailable        RPC_S_SERVER_UNAVAILABLE
#define rpc_s_call_orphaned             NOT_IMPLEMENTED+28
#define rpc_s_call_timeout              NOT_IMPLEMENTED+29
#define rpc_s_cant_recvmsg              NOT_IMPLEMENTED+30
#define rpc_s_connect_closed_by_rem     NOT_IMPLEMENTED+32
#define rpc_s_connect_rejected          NOT_IMPLEMENTED+33
#define rpc_s_connect_timed_out         NOT_IMPLEMENTED+34
#define rpc_s_fault_context_mismatch    NOT_IMPLEMENTED+36
#define rpc_s_host_unreachable          NOT_IMPLEMENTED+37
#define rpc_s_invalid_rpc_floor         NOT_IMPLEMENTED+39
#define rpc_s_loc_connect_aborted       NOT_IMPLEMENTED+40
#define rpc_s_network_unreachable       NOT_IMPLEMENTED+41
#define rpc_s_no_bindings               RPC_S_NO_MORE_BINDINGS
#define rpc_s_no_rem_endpoint           NOT_IMPLEMENTED+42
#define rpc_s_protocol_error            RPC_S_PROTOCOL_ERROR
#define rpc_s_rem_host_crashed          NOT_IMPLEMENTED+43
#define rpc_s_rem_host_down             NOT_IMPLEMENTED+44
#define rpc_s_rem_network_shutdown      NOT_IMPLEMENTED+45
#define rpc_s_rpcd_comm_failure         NOT_IMPLEMENTED+46
#define rpc_s_server_too_busy           RPC_S_SERVER_TOO_BUSY
#define rpc_s_socket_failure            NOT_IMPLEMENTED+47 
#define rpc_s_too_many_rem_connects     NOT_IMPLEMENTED+48
#define rpc_s_wrong_boot_time           NOT_IMPLEMENTED+49


#define rpc_s_call_cancelled             RPC_S_CALL_FAILED_DNE
#define rpc_s_call_faulted               NOT_IMPLEMENTED+50
#define rpc_s_fault_addr_error           NOT_IMPLEMENTED+51
#define rpc_s_fault_fp_div_by_zero       NOT_IMPLEMENTED+52
#define rpc_s_fault_fp_error             NOT_IMPLEMENTED+53
#define rpc_s_fault_fp_overflow          RPC_S_FP_OVERFLOW
#define rpc_s_fault_fp_underflow         RPC_S_FP_UNDERFLOW
#define rpc_s_fault_ill_inst             NOT_IMPLEMENTED+54
#define rpc_s_fault_int_div_by_zero      RPC_S_FP_DIV_ZERO
#define rpc_s_fault_int_overflow         NOT_IMPLEMENTED+55
#define rpc_s_fault_invalid_bound        RPC_S_INVALID_BOUND
#define rpc_s_fault_invalid_tag          NOT_IMPLEMENTED+56
#define rpc_s_fault_pipe_closed          NOT_IMPLEMENTED+57
#define rpc_s_fault_pipe_comm_error      NOT_IMPLEMENTED+58
#define rpc_s_fault_pipe_discipline      NOT_IMPLEMENTED+59
#define rpc_s_fault_pipe_empty           NOT_IMPLEMENTED+60
#define rpc_s_fault_pipe_memory          NOT_IMPLEMENTED+61
#define rpc_s_fault_pipe_order           NOT_IMPLEMENTED+62
#define rpc_s_fault_remote_comm_failure  NOT_IMPLEMENTED+63 
#define rpc_s_fault_remote_no_memory     NOT_IMPLEMENTED+64
#define rpc_s_fault_unspec               NOT_IMPLEMENTED+65
#define rpc_s_in_args_too_big            NOT_IMPLEMENTED+66
#define rpc_s_no_memory                  RPC_S_OUT_OF_MEMORY
 
#endif  // RPCMAP_H
